// -*- Mode: vala; indent-tabs-mode: nil; tab-width: 4 -*-
/***
    BEGIN LICENSE

    Copyright (C) 2011-2013 Wingpanel Developers
    This program is free software: you can redistribute it and/or modify it
    under the terms of the GNU Lesser General Public License version 3, as published
    by the Free Software Foundation.

    This program is distributed in the hope that it will be useful, but
    WITHOUT ANY WARRANTY; without even the implied warranties of
    MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
    PURPOSE.  See the GNU General Public License for more details.

    You should have received a copy of the GNU General Public License along
    with this program.  If not, see <http://www.gnu.org/licenses/>

    END LICENSE
***/

using SuperWingpanel;
using SuperWingpanel.Widgets;
using SuperWingpanel.Drawing;

namespace SuperWingpanel {
    
public class PanelController : GLib.Object
    {
        // Loads all indicators installed on the system
        private IndicatorLoader indicator_loader;

        // Read settings from dconf
        public Services.Settings settings;
        
        // The Gtk.Window
        public PanelWindow window;
        public PanelShadow shadow;

        public PositionManager position_manager;
        public PanelRenderer renderer;
        public HideManager hide_manager;

        public PanelController (WingpanelApp app)
        {
            settings = new Services.Settings ();
            indicator_loader = new Backend.IndicatorFactory (settings);
            

            window = new PanelWindow (app, this, indicator_loader);
            shadow = new PanelShadow ();

            position_manager = new PositionManager (this);
            renderer = new PanelRenderer (this);            
            hide_manager = new HideManager (this);

            hide_manager.initialize ();
            position_manager.initialize ();
            renderer.initialize ();
            
            window.show_all ();
            shadow.show_all ();

            settings.notify["hide-mode"].connect (set_struts);
            settings.notify["enable-slim-mode"].connect (set_struts);
            
            set_struts ();
        }



        private enum Struts {
            LEFT,
            RIGHT,
            TOP,
            BOTTOM,
            LEFT_START,
            LEFT_END,
            RIGHT_START,
            RIGHT_END,
            TOP_START,
            TOP_END,
            BOTTOM_START,
            BOTTOM_END,
            N_VALUES
        }

        public void set_struts () 
        {
            warning ("struts!");
            if (!window.get_realized ())
                return;

            int x, y, w, h;

            if (settings.hide_mode == HideType.NEVER_HIDE)
            {
                x = position_manager.X;
                y = position_manager.Y;

                w = position_manager.W;
                h = position_manager.H;
            }
            else
            {
                x = position_manager.monitor_geo.x;
                y = position_manager.monitor_geo.y;

                w = position_manager.monitor_geo.width;
                h = 0;
            }
                // Since uchar is 8 bits in vala but the struts are 32 bits
                // we have to allocate 4 times as much and do bit-masking
                var struts = new ulong[Struts.N_VALUES];

                struts[Struts.TOP] = h + y;
                struts[Struts.TOP_START] = x;
                struts[Struts.TOP_END] = x + w;

                var first_struts = new ulong[Struts.BOTTOM + 1];
                for (var i = 0; i < first_struts.length; i++)
                    first_struts[i] = struts[i];

                unowned X.Display display = Gdk.X11Display.get_xdisplay (window.get_display ());
                var xid = Gdk.X11Window.get_xid (window.get_window ());

                display.change_property (xid, display.intern_atom ("_NET_WM_STRUT_PARTIAL", false), X.XA_CARDINAL,
                                         32, X.PropMode.Replace, (uchar[]) struts, struts.length);
                display.change_property (xid, display.intern_atom ("_NET_WM_STRUT", false), X.XA_CARDINAL,
                                         32, X.PropMode.Replace, (uchar[]) first_struts, first_struts.length);
            

        }
    }
}